#pragma once

#include <sys/types.h>

#include "util.h"

struct blist_user {
  ssize_t id;
  unsigned char *name;
  ssize_t local_id;
};
typedef struct blist_user blist_user;

blist_user *blist_user_init();
blist_user *blist_user_init_from_sql(sqlite3_stmt *);
blist_user *blist_user_get_by_id(const blist *, u_int);
blist_user *blist_user_get_by_local(const blist *, u_int);
blist_user *blist_user_get_by_discord(const blist *, u_int);
blist_user *blist_user_get_by_google(const blist *, u_int);
void blist_user_save(const blist *, blist_user *);
void blist_user_delete(const blist *, const blist_user *);
void blist_user_deinit(blist_user *);

struct blist_list {
  ssize_t id;
  unsigned char *name;
  unsigned char *desc;
  bool is_preset;
};
typedef struct blist_list blist_list;

blist_list *blist_list_init();
blist_list *blist_list_init_from_sql(sqlite3_stmt *);
blist_list *blist_list_get_by_id(const blist *, u_int);
blist_list **blist_list_get_all_by_user(const blist *, u_int, size_t *);
blist_list *blist_list_get_by_user_and_name(const blist *, u_int, const char *);
void blist_list_save(const blist *, blist_list *, const blist_user *);
void blist_list_delete(const blist *, blist_list *);
void blist_list_deinit(blist_list *);

struct blist_stage {
  ssize_t id;
  unsigned char *name;
  unsigned char *desc;
  ssize_t list_id;
};
typedef struct blist_stage blist_stage;

blist_stage *blist_stage_init();
blist_stage *blist_stage_init_from_sql(sqlite3_stmt *);
blist_stage *blist_stage_get_by_id(const blist *, u_int);
blist_stage **blist_stage_get_all_for_list(const blist *, u_int, size_t *);
void blist_stage_save(const blist *, blist_stage *);
void blist_stage_delete(const blist *, const blist_stage *);
void blist_stage_deinit(blist_stage *);

struct blist_task {
  ssize_t id;
  unsigned char *name;
  unsigned char *desc;
  int stage_id;
  time_t due;
  int target_stage;
};
typedef struct blist_task blist_task;

blist_task *blist_task_init();
blist_task *blist_task_init_from_sql(sqlite3_stmt *);
blist_task *blist_task_get_by_id(const blist *, u_int);
blist_task **blist_task_get_for_list(const blist *, u_int, size_t *);
blist_task *blist_task_get_for_list_by_name(const blist *, u_int, const char *);
void blist_task_save(const blist *, blist_task *);
void blist_task_delete(const blist *, const blist_task *);
void blist_task_deinit(blist_task *);

void blist_print_user(blist *, const blist_user *);
void blist_print_list(const blist *, const blist_list *, blist_task **, size_t);
void blist_print_task(blist *, blist_task **, size_t);