#pragma once
#include <sqlite3.h>
#include <stdbool.h>
#include <stdio.h>
#include <sys/types.h>

/*
 * command parsing
 */

enum BLIST_LOG_LEVEL {
  BLIST_LOG_ERROR,
  BLIST_LOG_WARNING,
  BLIST_LOG_INFO,
  BLIST_LOG_DEBUG
};
typedef enum BLIST_LOG_LEVEL BLIST_LOG_LEVEL;

struct blist_logging_ctx {
  BLIST_LOG_LEVEL log_level;
  FILE *log_target;
};

typedef struct blist_logging_ctx blist_logging_ctx;

struct blist {
  sqlite3 *db;
  sqlite3_stmt **stmts;
  int error;
  blist_logging_ctx *log_ctx;
};
typedef struct blist blist;

blist *blist_init(char *db);
void blist_deinit(blist *list);

/*
 * sql preparations
 */

void blist_sql_prepare(blist *, char *);

/*
 * assuring things
 */

bool blist_assure_user(const blist *, __uid_t, const char *);
void blist_assure_list(blist *, char *);
void blist_assure_task(blist *, char *);

/*
 * printing methods
 *
 * json printing will be implemented eventually
 */

void blist_print(blist_logging_ctx *, BLIST_LOG_LEVEL, const char *, ...);